"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var logger_1 = require("../logger");
var ControllerUtils = (function () {
    function ControllerUtils(service) {
        this.service = service;
    }
    ControllerUtils.prototype.find = function (req, res, queryExecutor) {
        var sortBy = req.query.sort;
        var currentPage = req.query.page || 1;
        var perPage = req.query.per_page || 200;
        queryExecutor
            .execCount()
            .then(function (totalCount) {
            res.set('X-Total-Count', String(totalCount));
            return queryExecutor.execFind(sortBy, currentPage, perPage);
        })
            .then(this.respondWithResult(res))
            .catch(this.handleError(res));
    };
    ControllerUtils.prototype.all = function (req, res) {
        this.service
            .all()
            .then(this.respondWithResult(res))
            .catch(this.handleError(res));
    };
    ControllerUtils.prototype.byId = function (req, res) {
        this.service
            .byId(req.params.id)
            .then(this.handleEntityNotFound(res))
            .then(this.respondWithResult(res))
            .catch(this.handleError(res));
    };
    ControllerUtils.prototype.create = function (req, res) {
        this.service
            .create(req.body)
            .then(this.respondWithResult(res, 201))
            .catch(this.handleError(res));
    };
    ControllerUtils.prototype.update = function (req, res) {
        this.service
            .update(req.params.id, req.body)
            .then(this.handleEntityNotFound(res))
            .then(this.respondWithResult(res))
            .catch(this.handleError(res));
    };
    ControllerUtils.prototype.remove = function (req, res) {
        this.service
            .remove(req.params.id)
            .then(this.handleEntityNotFound(res))
            .then(this.respondWithResult(res))
            .catch(this.handleError(res));
    };
    ControllerUtils.prototype.respondWithResult = function (res, statusCode) {
        statusCode = statusCode || 200;
        return function (entity) {
            if (entity) {
                res.status(statusCode).json(entity);
            }
        };
    };
    ControllerUtils.prototype.handleEntityNotFound = function (res) {
        return function (entity) {
            if (!entity) {
                res.status(404).end();
                return null;
            }
            return entity;
        };
    };
    ControllerUtils.prototype.handleError = function (res, statusCode) {
        statusCode = statusCode || 500;
        return function (err) {
            logger_1.default.error(err);
            if (err.name === 'ValidationError')
                statusCode = 400;
            res.status(statusCode).send(err);
        };
    };
    return ControllerUtils;
}());
exports.ControllerUtils = ControllerUtils;
