"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var logger_1 = require("../logger");
var ServiceUtils = (function () {
    function ServiceUtils(model) {
        this.model = model;
    }
    ServiceUtils.prototype.all = function () {
        logger_1.default.info(this.model.collection.collectionName + ".all()");
        return this.model
            .find()
            .exec();
    };
    ServiceUtils.prototype.byId = function (id) {
        logger_1.default.info(this.model.collection.collectionName + ".byId(" + id + ")");
        return this.model
            .findById(id)
            .exec();
    };
    ServiceUtils.prototype.create = function (data) {
        logger_1.default.info(data, this.model.collection.collectionName + ".create(" + data + ")");
        return this.model.create(data);
    };
    ServiceUtils.prototype.update = function (id, data) {
        logger_1.default.info(data, this.model.collection.collectionName + ".update(" + data + ")");
        if (data._id)
            delete data._id;
        return this.model
            .findById(id)
            .exec()
            .then(this.saveUpdates(data));
    };
    ServiceUtils.prototype.remove = function (id) {
        logger_1.default.info(id, this.model.collection.collectionName + ".remove(" + id + ")");
        return this.model
            .findById(id)
            .exec()
            .then(this.removeEntity());
    };
    ServiceUtils.prototype.createQueryExecutor = function (queryBuilder) {
        var self = this;
        var executor = {};
        executor.execCount = function () {
            return self.execCount(queryBuilder);
        };
        executor.execFind = function (sortBy, currentPage, perPage) {
            return self.execFind(queryBuilder, sortBy, currentPage, perPage);
        };
        return executor;
    };
    //Query utils
    ServiceUtils.prototype.execFind = function (queryBuilder, sortBy, currentPage, perPage) {
        logger_1.default.info(queryBuilder, this.model.collection.collectionName + ".execFind()");
        var query = queryBuilder(this.model.find());
        if (sortBy)
            query.sort(sortBy);
        if (currentPage > 0 && perPage > 0)
            query
                .skip((currentPage - 1) * perPage)
                .limit(perPage);
        return query.exec();
    };
    ServiceUtils.prototype.execCount = function (queryBuilder) {
        logger_1.default.info(queryBuilder, this.model.collection.collectionName + ".execCount()");
        return queryBuilder(this.model.count(null)).exec();
    };
    //Query utils
    ServiceUtils.prototype.saveUpdates = function (updates) {
        return function (entity) {
            if (!entity)
                return;
            var updated = entity.merge(updates);
            return updated.save()
                .then(function (result) {
                return result;
            });
        };
    };
    ServiceUtils.prototype.removeEntity = function () {
        return function (entity) {
            if (!entity)
                return;
            return entity.remove()
                .then(function (removed) {
                return removed;
            });
        };
    };
    return ServiceUtils;
}());
exports.ServiceUtils = ServiceUtils;
